@@ -106,7 +106,7 @@ class Dataset(Attributable, DataContainer, Serializable, MetaDataListener):
         d = 'DATA'.center(width) + '\n' + '----'.center(width) + '\n'
         d += bstr(self.data, level=level, heavy=heavy, center=center,
                   tablefmt=tablefmt, tablefmt1=tablefmt1, tablefmt2=tablefmt2,
-                  **kwds) if matprint is None else \
+                  yaml=True, **kwds) if matprint is None else \
             matprint(self.data, level=level, trans=False, headers=[], tablefmt2='plain', heavy=heavy,
                      **kwds)
         return f'{s}\n{d}\n{last}\n'
@@ -124,8 +124,7 @@ class Dataset(Attributable, DataContainer, Serializable, MetaDataListener):
 
         s = OrderedDict(description=self.description,
                         meta=self._meta,
-                        data=self.data,
-                        _STID=self._STID)
+                        data=self.data)
         return s
 
 
@@ -225,5 +224,4 @@ class CompositeDataset(MetaDataListener, AbstractComposite):
         """
         return OrderedDict(  # description=self.description,
             _ATTR_meta=self._meta,
-            **self._data,
-            _STID=self._STID)
+            **self._data)
