@@ -12,7 +12,7 @@ logger = logging.getLogger(__name__)
 #logger.debug('level %d' %  (logger.getEffectiveLevel()))
 
 
-class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
+class DataContainer(Annotatable, Copyable, DeepEqual, Container):
     """ A DataContainer is a composite of data and description.
     mh: note that There is no metadata.
     Implemented partly from AbstractDataWrapper.
@@ -32,11 +32,11 @@ class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
         #print(__name__ + str(kwds))
 
         if data is None or issubclass(data.__class__, Container):
+            super().__init__(**kwds)  # DataContainer
             self.setData(data)
         else:
             raise TypeError('DataContainer needs a Container to initialize, not ' +
                             type(data).__name__)
-        super().__init__(**kwds)
 
     @property
     def data(self):
@@ -47,7 +47,7 @@ class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
         Returns
         -------
 
-        """ 
+        """
         return self.getData()
 
     @data.setter
@@ -85,14 +85,10 @@ class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
 
         Returns
         -------
-
+        `self._data` or `None` if '._data' is missing. e.g. subclass overriding `setData` calling `getData` from its `setData` that is called by 'this.__init__'.
         """
-        try:
-            return self._data
-        except AttributeError:
-            od = ODict()
-            self._data = od
-            return od
+
+        return getattr(self, '_data', None)
 
     def hasData(self):
         """ Returns whether this data wrapper has data. 
@@ -103,10 +99,10 @@ class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
 
         Returns
         -------
-
+        `False` if data does not exist or is `None` or empty, `True` if otherwise.
         """
-
-        return self.getData() is not None and len(self.getData()) > 0
+        d = getattr(self, '_data', None)
+        return bool(d)
 
     def __contains__(self, x):
         """
@@ -117,12 +113,12 @@ class DataContainer(Annotatable, Copyable, DeepEqual, Container, Sized):
             return False
 
     def __len__(self, *args, **kwargs):
-        """ size of data
+        """ 0 if data does not exist or is `None`, size of data if otherwise.
+
         """
-        try:
-            return self._data.__len__(*args, **kwargs)
-        except AttributeError:
-            return 0
+
+        d = getattr(self, '_data', None)
+        return len(d) if d else 0
 
 
 class DataWrapper(DataContainer, Quantifiable):
@@ -134,7 +130,7 @@ class DataWrapper(DataContainer, Quantifiable):
     def __init__(self, *args, **kwds):
         """ 
         """
-        super().__init__(*args, **kwds)
+        super().__init__(*args, **kwds)  # DataWrapper
 
 
 class DataWrapperMapper():
@@ -143,7 +139,7 @@ class DataWrapperMapper():
     def __init__(self, *args, **kwds):
         """ 
         """
-        super().__init__(*args, **kwds)
+        super().__init__(*args, **kwds)  # DataWrapperMapper
 
     def getDataWrappers(self):
         """ Gives the data wrappers, mapped by name.
@@ -155,4 +151,4 @@ class DataWrapperMapper():
         -------
 
         """
-        return self._sets
+        return self._data
