@@ -352,8 +352,7 @@ class AbstractParameter(Annotatable, Copyable, DeepEqual, DatasetEventSender, Se
         """
         return OrderedDict(description=self.description,
                            value=self.value,
-                           listeners=self.listeners,
-                           _STID=self._STID
+                           listeners=self.listeners
                            )
 
 
@@ -368,7 +367,7 @@ Seqs = (list, tuple, UserList)
 class Parameter(AbstractParameter, Typed):
     """ Parameter is the interface for all named attributes
     in the MetaData container. It can have a value and a description.
-    Defaul arguments: typ_='', default=None, valid=None.
+    Default arguments: typ_='', default=None, valid=None.
     value=default, description='UNKNOWN'
     """
 
@@ -417,7 +416,7 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
     def setType(self, typ_):
         """ Replaces the current type of this parameter.
 
-        Defaul will be casted if not the same.
+        Default will be casted if not the same.
         Unsuported parameter types will get a NotImplementedError.
         Parameters
         ----------
@@ -765,8 +764,7 @@ f        With two positional arguments: arg1-> value, arg2-> description. Parame
                            default=self._default,
                            value=self._value,  # must go behind type. maybe default
                            valid=self._valid,
-                           listeners=self.listeners,
-                           _STID=self._STID
+                           listeners=self.listeners
                            )
 
 
@@ -887,7 +885,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
         return r
 
     def toString(self, level=0,
-                 tablefmt='rst', tablefmt1='simple', tablefmt2='simple',
+                 tablefmt='grid', tablefmt1='simple', tablefmt2='simple',
                  param_widths=None, width=0, **kwds):
         """ return  string representation of metada.
 
@@ -959,7 +957,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
                 elif n == 'listeners' and len(v) == 0:
                     has_omission = True
                 else:
-                    ps = '%s=%s' % (att['name'], v.toString(level))
+                    ps = '%s=%s' % (n, v.toString(level)) if level == 2 else n
                     # tab.append(wls(ps, 80//N))
                     tab.append(ps)
 
@@ -979,7 +977,7 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
             fmt = tablefmt1
             s += tabulate(t, headers=headers, tablefmt=fmt, missingval='',
                           disable_numparse=True)
-        else:
+        elif level > 1:  # level 2 and 3
             s = ', '.join(tab) if len(tab) else 'Default Meta'
             l = '.'
             return '<' + self.__class__.__name__ + ' ' + s + l + '>'
@@ -1002,5 +1000,4 @@ class MetaData(ParameterListener, Composite, Copyable, DatasetEventSender):
         # print([id(o) for o in self.listeners])
 
         return OrderedDict(**self.data,
-                           _ATTR_listeners=self.listeners,
-                           _STID=self._STID)
+                           _ATTR_listeners=self.listeners)
