@@ -1,14 +1,16 @@
 # -*- coding: utf-8 -*-
 
 from .getswag import swag
-from .httppool_server import resp, excp, checkpath, unauthorized
-from ..model.user import auth
+from .httppool_server import resp, excp, checkpath, unauthorized, check_readonly
+from ..model.user import auth, getUsers
 from ...dataset.deserialize import deserialize_args
 from ...pal.poolmanager import PoolManager as PM, DEFAULT_MEM_POOL
+from ...pal.productpool import PoolNotFoundError
 from ...pal.webapi import WebAPI
+from ...pal.urn import parseUrn
 
-from flask import Blueprint, jsonify, request, current_app
-from flasgger import swag_from
+from flask import Blueprint, jsonify, request, current_app, url_for
+# from flasgger import swag_from
 
 import shutil
 import time
@@ -23,50 +25,100 @@ endp = swag['paths']
 
 pools_api = Blueprint('pools', __name__)
 
+
 ######################################
-#### /  get_registered_pools   ####
+####  /user/login POST  ####
 ######################################
 
 
-@ pools_api.route('/', methods=['GET'])
-# @ swag_from(endp['/']['get'])
-def get_registered_pools():
-    """ Returns a list of Pool IDs (pool names) of all pools registered with the Global PoolManager.
-    ---
+@ pools_api.route('/user/login', methods=['GET', 'POST'])
+@ auth.login_required(role=['read_only', 'read_write'])
+def login():
+    """ Logging in on the server.
+
+    :return: response made from http code, poolurl, message
     """
-    ts = time.time()
-    path = current_app.config['POOLPATH_BASE']
-    current_app.logger.debug('Listing all registered pools.')
 
-    result = [p.getPoolurl() for p in PM.getMap().values()]
-    msg = 'There is/are %d pools registered to the PoolManager.' % len(result)
-    code = 200
-    return resp(code, result, msg, ts)
-
-
-######################################
-#### /pools  get_pools/  reg/unreg ####
-######################################
-
-
-# @ pools_api.route('', methods=['GET'])
-@ pools_api.route('/pools/', methods=['GET'])
-def get_pools():
     logger = current_app.logger
-    if request.method in ['POST', 'PUT', 'DELETE'] and auth.current_user() == current_app.config['PC']['node']['ro_username']:
-        msg = 'User %s us Read-Only, not allowed to %s.' % \
-            (auth.current_user(), request.method)
-        logger.debug(msg)
-        return unauthorized(msg)
+    ts = time.time()
+    logger.debug('login')
+
+    msg = 'User %s logged-in %s.' % (auth.current_user().username,
+                                     auth.current_user().role)
+    logger.debug(msg)
+
+    return resp(200, 'OK', msg, ts, req_auth=True)
+
+######################################
+####  /user/logout POST  ####
+######################################
+
+
+@ pools_api.route('/user/logout', methods=['GET', 'POST'])
+@ auth.login_required(role=['read_only', 'read_write'])
+def logout():
+    """ Logging in on the server.
+
+    :return: response made from http code, poolurl, message
+    """
+
+    logger = current_app.logger
+    ts = time.time()
+    logger.debug('logout')
+
+    msg = 'User %s logged-out %s.' % (auth.current_user().username,
+                                      auth.current_user().role)
+    logger.debug(msg)
+
+    return resp(401, 'OK. Bye.', msg, ts)
+
+######################################
+#### /  get_pools   ####
+######################################
+
+
+@ pools_api.route('', methods=['GET'])
+def get_pools_url():
+    """ Get names of all pools, registered or not.
+    """
+    logger = current_app.logger
 
     ts = time.time()
     path = current_app.config['POOLPATH_BASE']
     logger.debug('Listing all directories from ' + path)
 
     result = get_name_all_pools(path)
+
+    if issubclass(result.__class__, list):
+        res = dict((x, request.base_url+'/'+x) for x in result)
+    else:
+        res = {}
     msg = '%d pools found.' % len(result)
     code = 200
-    return resp(code, result, msg, ts)
+    return resp(code, res, msg, ts)
+
+
+######################################
+#### /  get_pools   ####
+######################################
+
+
+@ pools_api.route('/', methods=['GET'])
+def get_pools():
+    """ Get names of all pools, registered or not.
+    """
+    logger = current_app.logger
+
+    ts = time.time()
+    path = current_app.config['POOLPATH_BASE']
+    logger.debug('Listing all directories from ' + path)
+
+    result = get_name_all_pools(path)
+
+    res = result
+    msg = '%d pools found.' % len(result)
+    code = 200
+    return resp(code, res, msg, ts)
 
 
 def get_name_all_pools(path):
@@ -84,28 +136,28 @@ def get_name_all_pools(path):
     current_app.logger.debug(path + ' has ' + str(alldirs))
     return alldirs
 
-
 ######################################
-####  user/login /logout GET  ####
+#### /pools  get_registered_pools/  ####
 ######################################
 
 
-@ pools_api.route('/user/login', methods=['GET'])
-@ auth.login_required
-def login():
-    """ Logging in on the server.
-
-    :return: response made from http code, poolurl, message
+# @ pools_api.route('', methods=['GET'])
+@ pools_api.route('/pools/', methods=['GET'])
+@ auth.login_required(role=['read_only', 'read_write'])
+def get_registered_pools():
+    """ Returns a list of Pool IDs (pool names) of all pools registered with the Global PoolManager.
+    ---
     """
-
-    logger = current_app.logger
-    __import__('pdb').set_trace()
-
-    msg = 'User %s logged-in Read-Write.' % auth.current_user().role
-    logger.debug(msg)
-
     ts = time.time()
-    return resp(200, 'OK', msg, ts)
+    path = current_app.config['POOLPATH_BASE']
+    current_app.logger.debug('Listing all registered pools.')
+
+    # [p.getPoolurl() for p in PM.getMap()()]
+    result = list(PM.getMap().keys())
+    msg = 'There is/are %d pools registered to the PoolManager.' % len(result)
+    code = 200
+    return resp(code, result, msg, ts)
+
 
 ######################################
 #### /pools/register_all pools/register_all/  ####
@@ -113,25 +165,30 @@ def login():
 
 
 @ pools_api.route('/pools/register_all', methods=['PUT'])
-# #@ swag_from(endp['/pools/register_all']['put'])
+@auth.login_required(role='read_write')
 def register_all():
     """ Register (Load) all pools on tme server.
 
 
     """
     ts = time.time()
-    pmap, bad = load_pools()
+
+    result, bad = load_pools(None, auth.current_user())
     code = 400 if len(bad) else 200
-    result = ', '.join(pmap.keys())
-    msg = '%d pools successfully loaded. Troubled: %s' % (len(pmap), str(bad))
+    # result = ', '.join(pmap.keys())
+    if issubclass(result.__class__, dict):
+        result = dict((x, request.base_url+x) for x in result)
+    msg = '%d pools successfully loaded. Troubled: %s' % (
+        len(result), str(bad))
     return resp(code, result, msg, ts)
 
 
-def load_pools(poolnames=None):
+def load_pools(poolnames, usr):
     """
     Adding all pool to server pool storage.
 
     poolnames: if given as a list of poolnames, only the exisiting ones of the list will be loaded.
+    :usr: current authorized user.
     Returns: a `dict` of successfully loaded pools names-pool in `good`, and troubled ones in `bad` with associated exception info.
     """
 
@@ -143,7 +200,7 @@ def load_pools(poolnames=None):
     alldirs = poolnames if poolnames else get_name_all_pools(path)
     for nm in alldirs:
         # must save the link or PM._GLOBALPOOLLIST will remove as dead weakref
-        code, thepool, msg = register_pool(nm)
+        code, thepool, msg = register_pool(nm, usr=usr)
         if code == 200:
             pmap[nm] = thepool
         else:
@@ -158,10 +215,13 @@ def load_pools(poolnames=None):
 
 
 @ pools_api.route('/pools/unregister_all', methods=['PUT'])
-# @ swag_from(endp['/pools/unregister_all']['delete'])
+@ auth.login_required(role='read_write')
 def unregister_all():
 
+    logger = current_app.logger
     ts = time.time()
+    logger.debug('unregister-all ' + pool)
+
     good, bad = unregister_pools()
     code = 200 if not bad else 416
     result = good
@@ -199,14 +259,17 @@ def unregister_pools(poolnames=None):
 
 
 @ pools_api.route('/pools/wipe_all', methods=['DELETE'])
-@ auth.login_required
+@ auth.login_required(role='read_write')
 def wipe_all():
     """ Remove contents of all pools.
 
     Only registerable pools will be wiped. Pool directories are not removed.
     """
+    logger = current_app.logger
     ts = time.time()
-    good, bad = wipe_pools()
+    logger.debug('Wipe-all pool')
+
+    good, bad = wipe_pools(None, auth.current_user())
     code = 200 if not bad else 416
     result = good
     msg = '%d pools wiped%s' % (len(good),
@@ -214,7 +277,7 @@ def wipe_all():
     return resp(code, result, msg, ts)
 
 
-def wipe_pools(poolnames=None):
+def wipe_pools(poolnames, usr):
     """
     Deleting all pools using pool api so locking is properly used.
 
@@ -230,7 +293,7 @@ def wipe_pools(poolnames=None):
 
     good = []
     notgood = []
-    all_pools, not_loadable = load_pools(poolnames)
+    all_pools, not_loadable = load_pools(poolnames, usr)
     names = list(all_pools.keys())
     for nm in copy.copy(names):
         thepool = all_pools[nm]
@@ -266,11 +329,11 @@ def get_pool(pool):
     ts = time.time()
     logger.debug('Get pool info of ' + pool)
 
-    _, result, _ = get_pool_info(pool)
+    result = get_pool_info(pool)
     return result
 
 
-def get_pool_info(poolname, serialize_out=True):
+def get_pool_info(poolname, serialize_out=False):
     ''' returns information of the pool.
     '''
     msg = ''
@@ -282,24 +345,41 @@ def get_pool_info(poolname, serialize_out=True):
         code, result, mes = load_HKdata([poolname], serialize_out=True)
         # use json.loads to avoid _STID for human
         result = json.loads(result)
-        print(result['classes'])
+        # move 'classes' to the last
+        c = result['classes']
+        del result['classes']
+        result['classes'] = c
+        # print(result['classes'])
         # for n in ['classes', 'urns', 'tags']:
         #    del result[n]['_STID']
-        for u in result['urns']:
-            del (result['urns'][u])['meta']
+        # Add url to tags
+        for t, ulist in result['tags'].items():
+            if t == '_STID':
+                continue
+            udict = {}
+            for u in ulist['urns']:
+                pn, cl, sn = parseUrn(u)
+                udict[u] = request.base_url + cl + '/' + str(sn)
+            ulist['urns'] = udict
+       # add urls to urns
+        for u, base in result['urns'].items():
+            if u == '_STID':
+                continue
+            base['meta'] = str(base['meta'])
+            pn, cl, sn = parseUrn(u)
+            base['url'] = request.base_url + cl + '/' + str(sn)
         msg = 'Getting pool %s information. %s.' % (poolname, mes)
     else:
-        code, result, msg = 404, FAILED, poolname +\
-            ' is not an exisiting Pool ID.'
-    return 0, resp(code, result, msg, ts, False), 0
+        code, result, msg = 404, FAILED, poolname + ' is not an exisiting Pool ID.'
+    return resp(code, result, msg, ts, False)
 
 
 ######################################
 ####  {pooolid}/register PUT /unreg DELETE  ####
 ######################################
 
-@auth.login_required
 @ pools_api.route('/<string:pool>', methods=['PUT'])
+@ auth.login_required(role='read_write')
 def register(pool):
     """ Register the given pool.
 
@@ -313,12 +393,12 @@ def register(pool):
     ts = time.time()
     logger.debug('register pool ' + pool)
 
-    code, thepool, msg = register_pool(pool)
+    code, thepool, msg = register_pool(pool, auth.current_user())
     res = thepool if issubclass(thepool.__class__, str) else thepool._poolurl
     return resp(code, res, msg, ts)
 
 
-def register_pool(pool):
+def register_pool(pool, usr):
     """ Register this pool to PoolManager.
 
     :returns: code, pool object if successful, message
@@ -326,10 +406,11 @@ def register_pool(pool):
     poolname = pool
     fullpoolpath = os.path.join(current_app.config['POOLPATH_BASE'], poolname)
     poolurl = current_app.config['POOLURL_BASE'] + poolname
+    makenew = usr and usr.role == 'read_write'
     try:
-        po = PM.getPool(poolname=poolname, poolurl=poolurl)
+        po = PM.getPool(poolname=poolname, poolurl=poolurl, makenew=makenew)
         return 200, po, 'register pool ' + poolname + ' OK.'
-    except (ValueError, NotImplementedError) as e:
+    except (ValueError, NotImplementedError, PoolNotFoundError) as e:
         code, result, msg = excp(
             e,
             msg='Unable to register pool: ' + poolname)
@@ -337,8 +418,8 @@ def register_pool(pool):
         return code, result, msg
 
 
-@ auth.login_required
 @ pools_api.route('/<string:pool>', methods=['DELETE'])
+@ auth.login_required(role='read_write')
 def unregister(pool):
     """ Unregister this pool from PoolManager.
 
@@ -346,12 +427,6 @@ def unregister(pool):
 
     """
     logger = current_app.logger
-    if auth.current_user() == current_app.config['PC']['node']['ro_username']:
-        msg = 'User %s us Read-Only, not allowed to %s.' % \
-            (auth.current_user(), request.method)
-        logger.debug(msg)
-        return unauthorized(msg)
-
     ts = time.time()
     logger.debug('Unregister pool ' + pool)
 
@@ -439,7 +514,6 @@ def load_HKdata(paths, serialize_out=True):
 
 
 @ pools_api.route('/<string:pool>/api/', methods=['GET'])
-# @ swag_from(endp['/{pool}']['put'])
 def api_info(pool):
     """ A list of names of allowed API methods.
 
@@ -460,7 +534,7 @@ def api_info(pool):
 
 
 @ pools_api.route('/<string:pool>/wipe', methods=['PUT'])
-@ auth.login_required
+@ auth.login_required(role='read_write')
 def wipe(pool):
     """ Removes all contents of the pool.
 
@@ -488,7 +562,6 @@ def wipe(pool):
 
 
 @ pools_api.route('/<string:pool>/hk/<string:kind>', methods=['GET'])
-# @ swag_from(endp['/{pool}']['put'])
 def hk_single(pool, kind):
     """ Returns the given kind of pool housekeeping data.
     """
@@ -580,7 +653,7 @@ def get_prod_count(prod_type, pool_id):
 
 @ pools_api.route('/<string:pool>/api/<string:method_args>', methods=['GET'])
 @ pools_api.route('/<string:pool>/api/<string:method_args>/', methods=['GET'])
-@ auth.login_required
+@ auth.login_required(role='read_write')
 def api(pool, method_args):
     """ Call api mathods on the running pool and returns the result.
 
@@ -612,7 +685,7 @@ def call_pool_Api(paths, serialize_out=False):
     args, kwds = [], {}
 
     # the unquoted args. may have ',' in strings
-    #quoted_m_args = paths[ind_meth+1]
+    # quoted_m_args = paths[ind_meth+1]
 
     # from the unquoted url extract the fist path segment.
     quoted_m_args = request.url.split(
