@@ -9,7 +9,7 @@ from .versionable import Versionable
 from .taggable import Taggable
 from .dicthk import DictHk
 from .definable import Definable
-from ..utils.common import pathjoin, fullname, lls
+from ..utils.common import pathjoin, fullname, lls, trbk
 from .productref import ProductRef
 from .query import AbstractQuery, MetaQuery, StorageQuery
 
@@ -275,7 +275,8 @@ When implementing a ProductPool, the following rules need to be applied:
 
         raise(NotImplementedError)
 
-    def schematicLoad(self, resourcetype, index, serialize_out=False):
+    def schematicLoad(self, resourcetype, index,
+                      start=None, end=None, serialize_out=False):
         """ to be implemented by subclasses to do the scheme-specific loading
         """
         raise(NotImplementedError)
@@ -294,7 +295,7 @@ When implementing a ProductPool, the following rules need to be applied:
             resourcetype=resource, index=index, serialize_out=serialize_out)
         return ret
 
-    def meta(self,  urn):
+    def meta(self, urn):
         """
         Loads the meta-data belonging to the product of specified URN.
         """
@@ -390,7 +391,12 @@ When implementing a ProductPool, the following rules need to be applied:
 ###########################
 
 
-MetaData_Json_Start = '{"_ATTR__meta": '
+class PoolNotFoundError(Exception):
+    pass
+
+
+# Do not include leading or trailing whitespace as they are not guarantteed.
+MetaData_Json_Start = '{"_ATTR_meta":'
 MetaData_Json_End = '"_STID": "MetaData"}'
 
 
@@ -439,7 +445,7 @@ class ManagedPool(ProductPool, DictHk):
         """
         return makeLockpath(self.transformpath(self._poolname), op)
 
-    @ lru_cache(maxsize=5)
+    @ lru_cache(maxsize=32)
     def transformpath(self, path):
         """ override this to changes the output from the input one (default) to something else.
 
@@ -454,6 +460,13 @@ class ManagedPool(ProductPool, DictHk):
                 path = base + '/' + path
         return path
 
+    def getCacheInfo(self):
+        info = {}
+        for i in ['transformpath']:
+            info[i] = getattr(self, i).cache_info()
+
+        return info
+
     def dereference(self, ref):
         """
         Decrement the reference count of a ProductRef.
@@ -506,11 +519,31 @@ class ManagedPool(ProductPool, DictHk):
         """
         return self._urns[urn]
 
+    def setMetaByUrn(self, start, end, urn):
+        """
+        Sets the location of the meta data of the specified data to the given URN.
+
+        :data: usually un/serialized Product.
+        """
+        u = urn.urn if issubclass(urn.__class__, Urn) else urn
+        if u not in self._urns:
+            raise ValueError(urn + ' not found in pool ' + self._poolname)
+
+        raise NotImplemented
+
+    def getMetaByUrn(self, urn, resourcetype=None, index=None):
+        """ 
+        Get all of the meta data belonging to a product of a given URN.
+
+        mh: returns an iterator.
+        """
+        raise NotImplemented
+
     def meta(self,  urn):
         """
-        Loads the meta-data belonging to the product of specified URN.
+        Loads the meta-data info belonging to the product of specified URN.
         """
-        return self._urns[urn]['meta']
+        return self.getMetaByUrn(urn)
 
     def reference(self, ref):
         """
@@ -554,17 +587,7 @@ class ManagedPool(ProductPool, DictHk):
             urn = makeUrn(poolname=self._poolname, typename=pn, index=sn)
 
             if urn not in u:
-                if serialize_in and hasattr(prd, 'meta'):
-                    mt = prd.meta
-                else:
-                    start = prd.find(MetaData_Json_Start, 0)
-                    end = prd.find(MetaData_Json_End, start)
-                    if start >= 0 and end > 0:
-                        mt = deserialize(
-                            prd[start+len(MetaData_Json_Start):end+len(MetaData_Json_End)])
-                    else:
-                        mt = ODict()
-                u[urn] = dict(tags=[], meta=mt)
+                u[urn] = dict(tags=[])
 
             if tag is not None:
                 self.setTag(tag, urn)
@@ -577,11 +600,11 @@ class ManagedPool(ProductPool, DictHk):
                             serialize_in=serialize_in,
                             serialize_out=serialize_out,
                             **kwds)
-            except Exception as e:
-                msg = 'product ' + urn + ' saving failed.'
+            except ValueError as e:
+                msg = 'product ' + urn + ' saving failed.' + str(e) + trbk(e)
+                logger.debug(msg)
                 self._classes, self._tags, self._urns = tuple(
                     self.readHK().values())
-                logger.debug(msg)
                 raise e
 
         if geturnobjs:
@@ -648,18 +671,20 @@ class ManagedPool(ProductPool, DictHk):
         else:
             return serialize(res[0]) if serialize_out else res[0]
 
-    def doLoad(self, resourcetype, index, serialize_out=False):
+    def doLoad(self, resourcetype, index, start=None, end=None, serialize_out=False):
         """ to be implemented by subclasses to do the action of loading
         """
         raise(NotImplementedError)
 
-    def schematicLoad(self, resourcetype, index, serialize_out=False):
+    def schematicLoad(self, resourcetype, index, start=None, end=None,
+                      serialize_out=False):
         """ do the scheme-specific loading
         """
 
         with filelock.FileLock(self.lockpath('w')):
             ret = self.doLoad(resourcetype=resourcetype,
-                              index=index, serialize_out=serialize_out)
+                              index=index, start=start, end=end,
+                              serialize_out=serialize_out)
         return ret
 
     def doRemove(self, resourcetype, index):
@@ -712,10 +737,10 @@ class ManagedPool(ProductPool, DictHk):
         with filelock.FileLock(self.lockpath('w')),\
                 filelock.FileLock(self.lockpath('r')):
             try:
-                self.doWipe()
                 self._classes.clear()
                 self._tags.clear()
                 self._urns.clear()
+                self.doWipe()
             except Exception as e:
                 msg = self.getId() + 'wiping failed'
                 logger.debug(msg)
@@ -739,14 +764,14 @@ class ManagedPool(ProductPool, DictHk):
                 code = compile(qw, 'py', 'eval')
                 for ref in reflist:
                     refmet = ref.getMeta()
-                    m = refmet if refmet else u[ref.urn]['meta']
+                    m = refmet if refmet else self.getMetaByUrn(ref.urn)
                     if eval(code):
                         ret.append(ref)
                 return ret
             else:
                 for ref in reflist:
                     refmet = ref.getMeta()
-                    m = refmet if refmet else u[ref.urn]['meta']
+                    m = refmet if refmet else self.getMetaByUrn(ref.urn)
                     if qw(m):
                         ret.append(ref)
                 return ret
@@ -754,13 +779,13 @@ class ManagedPool(ProductPool, DictHk):
             if isinstance(qw, str):
                 code = compile(qw, 'py', 'eval')
                 for urn in urnlist:
-                    m = u[urn]['meta']
+                    m = self.getMetaByUrn(urn)
                     if eval(code):
                         ret.append(ProductRef(urn=urn, meta=m))
                 return ret
             else:
                 for urn in urnlist:
-                    m = u[urn]['meta']
+                    m = self.getMetaByUrn(urn)
                     if qw(m):
                         ret.append(ProductRef(urn=urn, meta=m))
                 return ret
@@ -770,7 +795,7 @@ class ManagedPool(ProductPool, DictHk):
                 for n in snlist:
                     urn = makeUrn(poolname=self._poolname,
                                   typename=typename, index=n)
-                    m = u[urn]['meta']
+                    m = self.getMetaByUrn(urn)
                     if eval(code):
                         ret.append(ProductRef(urn=urn, meta=m))
                 return ret
@@ -778,7 +803,7 @@ class ManagedPool(ProductPool, DictHk):
                 for n in snlist:
                     urn = makeUrn(poolname=self._poolname,
                                   typename=typename, index=n)
-                    m = u[urn]['meta']
+                    m = self.getMetaByUrn(urn)
                     if qw(m):
                         ret.append(ProductRef(urn=urn, meta=m))
                 return ret
