@@ -6,7 +6,7 @@ from fdi.dataset.numericparameter import NumericParameter
 from fdi.dataset.stringparameter import StringParameter
 from fdi.dataset.eq import deepcmp
 from fdi.dataset.deserialize import serialize_args, deserialize_args
-from fdi.dataset.testproducts import get_sample_product
+from fdi.dataset.testproducts import get_demo_product
 from fdi.pal.productstorage import ProductStorage
 from fdi.pal.query import MetaQuery
 from fdi.pal.poolmanager import PoolManager, DEFAULT_MEM_POOL
@@ -51,7 +51,7 @@ def init_test():
 
 
 def chksa(a, k):
-    p = 1
+    p = 0
     s = serialize_args(*a, **k)
     if p:
         print('s= ', s)
@@ -224,7 +224,7 @@ def crud_t(poolid, poolurl, local_pools_dir, pool):
 
     tag = '==Demo ** Product=='
     logger.info('test sample demo prod with tag: '+tag)
-    sp = get_sample_product()
+    sp = get_demo_product()
     urn = pstore.save(sp, tag=tag)
     print('Sample Prod saved with tag "%s" %s to %s' %
           (tag, urn.urn, pool.poolname))
