@@ -51,7 +51,7 @@ from fdi.dataset.product import Product
 from fdi.dataset.browseproduct import BrowseProduct
 from fdi.dataset.readonlydict import ReadOnlyDict
 from fdi.dataset.unstructureddataset import UnstrcturedDataset
-from fdi.dataset.testproducts import SP, get_sample_product
+from fdi.dataset.testproducts import SP, get_demo_product
 from fdi.utils.checkjson import checkjson
 from fdi.utils.loadfiles import loadMedia
 from fdi.utils.ydump import ydump
@@ -2086,28 +2086,28 @@ def test_UnstrcturedDataset():
     assert u.doctype is None
     u.doctype = 'json'
     assert u.doctype == 'json'
-    p = get_sample_product()
+
+    p = get_demo_product()
     u.input(p.serialized(), 'json')
     assert issubclass(u.data.__class__, dict)
 
-    v, s = u.fetch(["data", "_ATTR__meta", "speed"])
+    v, s = u.fetch(["data", "_ATTR_meta", "speed"])
     # v is a dictionary
     assert json.dumps(v) == serialize(p.meta['speed'])
-    assert s == '.data["_ATTR__meta"]["speed"]'
+    assert s == '.data["_ATTR_meta"]["speed"]'
     # ["results"]["calibration"]["meta"]["unit"]["value"]
     v, s = u.fetch(["data", "results", "calibration",
-                    "_ATTR__meta", "unit", "value"])
+                    "_ATTR_meta", "unit", "value"])
     assert v == 'count'
-    assert s == '.data["results"]["calibration"]["_ATTR__meta"]["unit"]["value"]'
+    assert s == '.data["results"]["calibration"]["_ATTR_meta"]["unit"]["value"]'
     # data of a column in tabledataset within compositedataset
     v, s = u.fetch(
         ["data", "results", "Time_Energy_Pos", "Energy", "_ATTR_data"])
-    t = [x * 1.0 for x in range(9)]
+    t = [x * 1.0 for x in range(len(v))]
     assert v == [2 * x + 100 for x in t]
     assert v == u.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]
     assert s == '.data["results"]["Time_Energy_Pos"]["Energy"]["_ATTR_data"]'
 
-    # print(u.toString())
     checkjson(u)
     checkgeneral(u)
 
@@ -2145,6 +2145,25 @@ bookstore = """{ "store": {
   }
 }"""
 
+# http://omz-software.com/pythonista/docs/ios/xmltodict.html
+simple_ex = """
+<a prop="x">
+   <b>1</b>
+   <b>2</b>
+</a>
+"""
+complex_ex = """
+<mydocument has="an attribute">
+  <and>
+    <many>elements</many>
+    <many>more elements</many>
+  </and>
+  <plus a="complex">
+    element as well
+  </plus>
+</mydocument>
+"""
+
 
 def test_jsonPath():
     js = '[{"idAssetType":6,"name":"CCAA","enterprise":{"id":1,"name":"APV"}}]'
@@ -2152,8 +2171,17 @@ def test_jsonPath():
     m = u.jsonPath("$[?(@.name == 'CCAA')].idAssetType")
     assert m[0][1] == 6
 
+    # xmltodict docs
+    u = UnstrcturedDataset(data=simple_ex, doctype='xml', attr_prefix='@')
+    assert u.data['a']['@prop'] == 'x'
+    assert u.data['a']['b'] == ['1', '2']
+    u.attr_prefix = '%'
+    u.input(simple_ex)
+    assert u.data['a']['%prop'] == 'x'
+
     ### BOOK STORE ###
     u = UnstrcturedDataset(data=bookstore, doctype='json')
+    # print(u.toString())
     # the authors of all books in the store
     n = u.jsonPath("$.store.book[*].author", val='context')
     assert list(x.value for x in n) == [
@@ -2197,6 +2225,10 @@ def test_jsonPath():
     with pytest.raises(AssertionError):
         assert len(m) > 0
     # use this instead. MUST HAVE THE WHITESPACES in ' - 1'
+    # NOT working: Full diff:
+    #- ['J. R. R. Tolkien']
+    #+ ['Nigel Rees', 'Evelyn Waugh', 'Herman Melville', 'J. R. R. Tolkien']
+    #
     #m = u.jsonPath('$..book[?(@.`len` - 1)]', val='context')
     #assert list(x.value['author'] for x in m) == ['J. R. R. Tolkien']
     # the last book in order. #2
@@ -2565,13 +2597,6 @@ def test_FineTimes_toString():
         assert ts == out_FineTime
 
 
-def xtest_get_sample_product():
-    v = get_sample_product()
-    assert v['Browse'].data[1:4] == b'PNG'
-    checkjson(v)
-    checkgeneral(v)
-
-
 def test_History():
     v = History()
     checkjson(v)
