@@ -265,7 +265,8 @@ def cleanup(poolurl=None, poolname=None):
             d = direc + '/' + pn
             rmlocal(d)
         elif schm == 'mem':
-            pass
+            if PoolManager.isLoaded(DEFAULT_MEM_POOL):
+                PoolManager.getPool(DEFAULT_MEM_POOL).removeAll()
         elif schm in ['http', 'https']:
             pass
         else:
@@ -405,16 +406,21 @@ def test_ProductRef():
     p = s + '/' + a3  # a pool URL
     r = a4 + ':' + str(a5)  # a resource
     u = 'urn:' + a3 + ':' + r    # a URN
-    cleanup(p, u)
+    cleanup(p, a3)
+
+    cleanup('mem:///' + DEFAULT_MEM_POOL, DEFAULT_MEM_POOL)
 
     # in memory
     # A productref created from a single product will result in a memory pool urn, and the metadata won't be loaded.
     v = ProductRef(prd)
     # only one prod in memory pool
-    checkdbcount(1, 'mem:///'+DEFAULT_MEM_POOL, a4, 0, '')
-    assert v.urn == 'urn:'+DEFAULT_MEM_POOL+':' + a4 + ':' + str(0)
+    checkdbcount(1, 'mem:///' + DEFAULT_MEM_POOL, a4, 0, '')
+    assert v.urn == 'urn:' + DEFAULT_MEM_POOL + ':' + a4 + ':' + str(0)
     assert v.meta is None
     assert v.product == prd
+    # run again will get a different urn from the new instance in the pool
+    r = ProductRef(prd)
+    assert v != r
 
     # construction
 
@@ -580,6 +586,9 @@ def check_ps_func_for_pool(thepoolname, thepoolurl, *args):
     checkdbcount(n - 1, thepoolurl, pcq, n, *args)
     checkdbcount(1, thepoolurl, fullname(MapContext), 0, *args)
 
+    # report lru_cache info
+    print('****** %s ' % str(type(pspool)), pspool.getCacheInfo())
+
     # clean up a pool
     ps.wipePool()
     checkdbcount(0, thepoolurl, pcq, -1, *args)
@@ -876,6 +885,9 @@ def doquery(poolpath, newpoolpath):
     chk(res[0], rec1[3])
     chk(res[1], rec1[4])
 
+    # report lru_cache info
+    print('****** %s ' % str(type(thepool)), thepool.getCacheInfo())
+
 
 def test_query_local_mem():
     cleanup()
@@ -1034,7 +1046,7 @@ def test_MapContext():
     # the new and the old one are equal
     assert newp2 == x
 
-    des = checkjson(mc)
+    des = checkjson(mc, dbg=0)
     # print(type(des['refs']))
     # print('&&&&&& ' + des.refs.serialized(indent=4) + ' %%%%%%')
     # print(yaml.dump(des))
